



pytest -vx --doctest-modules --pyargs datashape -k "not (test_arbitrary_string or test_validate or test_nested_iteratables or test_tuples_can_be_records_too)"
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
